@@ -7,10 +7,8 @@ from unittest import mock
 
 import pytest
 
-import pypiserver.bottle
+import pypiserver.bottle_wrapper as bottle
 from pypiserver import __main__
-from pypiserver.bottle import Bottle
-
 
 THIS_DIR = pathlib.Path(__file__).parent
 HTPASS_FILE = THIS_DIR / "../fixtures/htpasswd.a.a"
@@ -18,7 +16,7 @@ IGNORELIST_FILE = THIS_DIR / "test-ignorelist"
 
 
 class main_wrapper:
-    app: t.Optional[Bottle]
+    app: t.Optional[bottle.Bottle]
     run_kwargs: t.Optional[dict]
     update_args: t.Optional[tuple]
     update_kwargs: t.Optional[dict]
@@ -51,7 +49,7 @@ def main(monkeypatch):
         main.update_args = args
         main.update_kwargs = kwargs
 
-    monkeypatch.setattr("pypiserver.bottle.run", run)
+    monkeypatch.setattr("pypiserver.bottle_wrapper.run", run)
     monkeypatch.setattr("pypiserver.manage.update_all_packages", update)
 
     return main
@@ -262,7 +260,7 @@ def test_auto_servers() -> None:
     """Test auto servers."""
     # A list of bottle ServerAdapters
     bottle_adapters = tuple(
-        a.__name__.lower() for a in pypiserver.bottle.AutoServer.adapters
+        a.__name__.lower() for a in bottle.AutoServer.adapters
     )
     # We are going to expect that our AutoServer enum names must match those
     # at least closely enough to be recognizable.
